@@ -7,10 +7,9 @@
 
 #define G_LOG_DOMAIN "callaudiod-pulse"
 
+#include "cad-manager.h"
 #include "cad-pulse.h"
 
-#include "libcallaudio.h"
-
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <pulse/pulseaudio.h>
@@ -33,6 +32,8 @@ struct _CadPulse
 {
     GObject parent_instance;
 
+    GObject *manager;
+
     pa_glib_mainloop  *loop;
     pa_context        *ctx;
 
@@ -42,11 +43,14 @@ struct _CadPulse
 
     gboolean has_voice_profile;
     gchar *speaker_port;
+    gchar *earpiece_port;
 
     GHashTable *sink_ports;
     GHashTable *source_ports;
 
-    CallAudioMode current_mode;
+    CallAudioMode audio_mode;
+    CallAudioSpeakerState speaker_state;
+    CallAudioMicState mic_state;
 };
 
 G_DEFINE_TYPE(CadPulse, cad_pulse, G_TYPE_OBJECT);
@@ -149,7 +153,7 @@ static void process_new_source(CadPulse *self, const pa_source_info *info)
     prop = pa_proplist_gets(info->proplist, PA_PROP_DEVICE_CLASS);
     if (prop && strcmp(prop, SINK_CLASS) != 0)
         return;
-    if (g_str_has_suffix(info->name, "monitor"))
+    if (info->monitor_of_sink != PA_INVALID_INDEX)
         return;
     if (info->card != self->card_id || self->source_id != -1)
         return;
@@ -194,6 +198,14 @@ static void init_source_info(pa_context *ctx, const pa_source_info *info, int eo
     if (op)
         pa_operation_unref(op);
 
+    if (self->mic_state == CALL_AUDIO_MIC_UNKNOWN) {
+        if (info->mute)
+            self->mic_state = CALL_AUDIO_MIC_OFF;
+        else
+            self->mic_state = CALL_AUDIO_MIC_ON;
+        g_object_set(self->manager, "mic-state", self->mic_state, NULL);
+    }
+
     target_port = get_available_source_port(info, NULL);
     if (target_port) {
         op = pa_context_set_source_port_by_index(ctx, self->source_id,
@@ -313,6 +325,15 @@ static void process_new_sink(CadPulse *self, const pa_sink_info *info)
             } else {
                 self->speaker_port = g_strdup(port->name);
             }
+        } else if (strstr(port->name, SND_USE_CASE_DEV_EARPIECE) != NULL) {
+            if (self->earpiece_port) {
+                if (strcmp(port->name, self->earpiece_port) != 0) {
+                    g_free(self->earpiece_port);
+                    self->earpiece_port = g_strdup(port->name);
+                }
+            } else {
+                self->earpiece_port = g_strdup(port->name);
+            }
         }
 
         if (port->available != PA_PORT_AVAILABLE_UNKNOWN) {
@@ -323,6 +344,7 @@ static void process_new_sink(CadPulse *self, const pa_sink_info *info)
     }
 
     g_debug("SINK:   speaker_port='%s'", self->speaker_port);
+    g_debug("SINK:   earpiece_port='%s'", self->earpiece_port);
 }
 
 static void init_sink_info(pa_context *ctx, const pa_sink_info *info, int eol, void *data)
@@ -347,6 +369,54 @@ static void init_sink_info(pa_context *ctx, const pa_sink_info *info, int eol, v
     if (op)
         pa_operation_unref(op);
 
+    if (self->speaker_state == CALL_AUDIO_SPEAKER_UNKNOWN) {
+        self->speaker_state = CALL_AUDIO_SPEAKER_OFF;
+
+        switch (self->audio_mode) {
+        case CALL_AUDIO_MODE_CALL:
+            if (g_strcmp0(info->active_port->name, self->speaker_port) == 0) {
+                self->speaker_state = CALL_AUDIO_SPEAKER_ON;
+                g_object_set(self->manager, "speaker-state", self->speaker_state, NULL);
+                /*
+                 * callaudiod likely restarted after being killed during a call
+                 * during which the speaker was enabled. End processing here so
+                 * we keep the current routing and don't unexpectedly mess with
+                 * the call setup.
+                 */
+                return;
+            }
+            break;
+        case CALL_AUDIO_MODE_UNKNOWN:
+            /*
+             * Let's see if we can guess the current mode:
+             * - if current port is earpiece, we're likely in call mode
+             * - otherwise we're either in default mode, or call mode with
+             *   speaker enabled. Let's settle for the former as both situations
+             *   are technically equivalent.
+             *
+             * Note: this code path is only used when the card doesn't have a
+             * voice profile, otherwise things are easier to deal with.
+             */
+            if (g_strcmp0(info->active_port->name, self->earpiece_port) == 0) {
+                self->audio_mode = CALL_AUDIO_MODE_CALL;
+                g_object_set(self->manager, "audio-mode", self->audio_mode, NULL);
+                /*
+                 * Don't touch routing as we're likely in the middle of a call,
+                 * see above.
+                 */
+                return;
+            } else {
+                self->audio_mode = CALL_AUDIO_MODE_DEFAULT;
+                g_object_set(self->manager, "audio-mode", self->audio_mode, NULL);
+            }
+            break;
+        default:
+            break;
+        }
+
+        g_object_set(self->manager, "speaker-state", self->speaker_state, NULL);
+    }
+
     target_port = get_available_sink_port(info, NULL);
     if (target_port) {
         g_debug("  Using sink port '%s'", target_port);
@@ -425,10 +495,18 @@ static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, v
 
         if (strstr(profile->name, SND_USE_CASE_VERB_VOICECALL) != NULL) {
             self->has_voice_profile = TRUE;
+            if (info->active_profile2 == profile)
+                self->audio_mode = CALL_AUDIO_MODE_CALL;
+            else
+                self->audio_mode = CALL_AUDIO_MODE_DEFAULT;
             break;
         }
     }
 
+    // We were able determine the current mode, set the corresponding D-Bus property
+    if (self->audio_mode != CALL_AUDIO_MODE_UNKNOWN)
+        g_object_set(self->manager, "audio-mode", self->audio_mode, NULL);
+
     g_debug("CARD:   %s voice profile", self->has_voice_profile ? "has" : "doesn't have");
 
     /* Found a suitable card, let's prepare the sink/source */
@@ -632,6 +710,8 @@ static void dispose(GObject *object)
 
     if (self->speaker_port)
         g_free(self->speaker_port);
+    if (self->earpiece_port)
+        g_free(self->earpiece_port);
 
     pulseaudio_cleanup(self);
 
@@ -653,6 +733,10 @@ static void cad_pulse_class_init(CadPulseClass *klass)
 
 static void cad_pulse_init(CadPulse *self)
 {
+    self->manager = G_OBJECT(cad_manager_get_default());
+    self->audio_mode = CALL_AUDIO_MODE_UNKNOWN;
+    self->speaker_state = CALL_AUDIO_SPEAKER_UNKNOWN;
+    self->mic_state = CALL_AUDIO_MIC_UNKNOWN;
 }
 
 CadPulse *cad_pulse_get_default(void)
@@ -684,12 +768,42 @@ static void operation_complete_cb(pa_context *ctx, int success, void *data)
     if (operation) {
         if (operation->op) {
             operation->op->success = (gboolean)!!success;
-            operation->op->callback(operation->op);
+            if (operation->op->callback)
+                operation->op->callback(operation->op);
 
-            if (operation->op->type == CAD_OPERATION_SELECT_MODE &&
-                operation->op->success) {
-                operation->pulse->current_mode = operation->value;
+            if (operation->op->success) {
+                guint new_value = GPOINTER_TO_UINT(operation->value);
+
+                switch (operation->op->type) {
+                case CAD_OPERATION_SELECT_MODE:
+                    if (operation->pulse->audio_mode != new_value) {
+                        operation->pulse->audio_mode = new_value;
+                        g_object_set(operation->pulse->manager, "audio-mode", new_value, NULL);
+                    }
+                    break;
+                case CAD_OPERATION_ENABLE_SPEAKER:
+                    if (operation->pulse->speaker_state != new_value) {
+                        operation->pulse->speaker_state = new_value;
+                        g_object_set(operation->pulse->manager, "speaker-state", new_value, NULL);
+                    }
+                    break;
+                case CAD_OPERATION_MUTE_MIC:
+                    /*
+                     * "Mute mic" operation's value is TRUE (1) for muting the mic,
+                     * so ensure mic_state carries the right value.
+                     */
+                    new_value = new_value ? CALL_AUDIO_MIC_OFF : CALL_AUDIO_MIC_ON;
+                    if (operation->pulse->mic_state != new_value) {
+                        operation->pulse->mic_state = new_value;
+                        g_object_set(operation->pulse->manager, "mic-state", new_value, NULL);
+                    }
+                    break;
+                default:
+                    break;
+                }
             }
+
+            free(operation->op);
         }
 
         free(operation);
@@ -752,7 +866,7 @@ static void set_output_port(pa_context *ctx, const pa_sink_info *info, int eol,
     if (info->card != operation->pulse->card_id || info->index != operation->pulse->sink_id)
         return;
 
-    if (operation->op->type == CAD_OPERATION_SELECT_MODE) {
+    if (operation->op && operation->op->type == CAD_OPERATION_SELECT_MODE) {
         /*
          * When switching to voice call mode, we want to switch to any port
          * other than the speaker; this makes sure we use the headphones if they
@@ -794,40 +908,6 @@ static void set_output_port(pa_context *ctx, const pa_sink_info *info, int eol,
     }
 }
 
-static void set_mic_mute(pa_context *ctx, const pa_source_info *info, int eol, void *data)
-{
-    CadPulseOperation *operation = data;
-    pa_operation *op = NULL;
-
-    if (eol != 0)
-        return;
-
-    if (!info) {
-        g_critical("PA returned no source info (eol=%d)", eol);
-        return;
-    }
-
-    if (info->card != operation->pulse->card_id || info->index != operation->pulse->source_id)
-        return;
-
-    if (info->mute && !operation->value) {
-        g_debug("mic is muted, unmuting...");
-        op = pa_context_set_source_mute_by_index(ctx, operation->pulse->source_id, 0,
-                                                 operation_complete_cb, operation);
-    } else if (!info->mute && operation->value) {
-        g_debug("mic is active, muting...");
-        op = pa_context_set_source_mute_by_index(ctx, operation->pulse->source_id, 1,
-                                                 operation_complete_cb, operation);
-    }
-
-    if (op) {
-        pa_operation_unref(op);
-    } else {
-        g_debug("%s: nothing to be done", __func__);
-        operation_complete_cb(ctx, 1, operation);
-    }
-}
-
 /**
  * cad_pulse_select_mode:
  * @mode:
@@ -861,16 +941,21 @@ void cad_pulse_select_mode(guint mode, CadOperation *cad_op)
         /*
          * When ending a call, we want to make sure the mic doesn't stay muted
          */
-        CadPulseOperation *unmute_op = g_new0(CadPulseOperation, 1);
+        CadOperation *unmute_op = g_new0(CadOperation, 1);
+        unmute_op->type = CAD_OPERATION_MUTE_MIC;
 
-        unmute_op->pulse = operation->pulse;
-        unmute_op->value = FALSE;
+        cad_pulse_mute_mic(FALSE, unmute_op);
 
-        op = pa_context_get_source_info_by_index(unmute_op->pulse->ctx,
-                                                 unmute_op->pulse->source_id,
-                                                 set_mic_mute, unmute_op);
-        if (op)
-            pa_operation_unref(op);
+        /*
+         * If the card has a dedicated voice profile, disable speaker so it
+         * doesn't get automatically enabled for next call.
+         */
+        if (operation->pulse->has_voice_profile) {
+            CadOperation *disable_speaker_op = g_new0(CadOperation, 1);
+            disable_speaker_op->type = CAD_OPERATION_ENABLE_SPEAKER;
+
+            cad_pulse_enable_speaker(FALSE, disable_speaker_op);
+        }
     }
 
     if (operation->pulse->has_voice_profile) {
@@ -982,11 +1067,24 @@ void cad_pulse_mute_mic(gboolean mute, CadOperation *cad_op)
     operation->op = cad_op;
     operation->value = (guint)mute;
 
-    op = pa_context_get_source_info_by_index(operation->pulse->ctx,
-                                             operation->pulse->source_id,
-                                             set_mic_mute, operation);
-    if (op)
+    if (operation->pulse->mic_state == CALL_AUDIO_MIC_OFF && !operation->value) {
+        g_debug("mic is muted, unmuting...");
+        op = pa_context_set_source_mute_by_index(operation->pulse->ctx,
+                                                 operation->pulse->source_id, 0,
+                                                 operation_complete_cb, operation);
+    } else if (operation->pulse->mic_state == CALL_AUDIO_MIC_ON && operation->value) {
+        g_debug("mic is active, muting...");
+        op = pa_context_set_source_mute_by_index(operation->pulse->ctx,
+                                                 operation->pulse->source_id, 1,
+                                                 operation_complete_cb, operation);
+    }
+
+    if (op) {
         pa_operation_unref(op);
+    } else {
+        g_debug("%s: nothing to be done", __func__);
+        operation_complete_cb(operation->pulse->ctx, 1, operation);
+    }
 
     return;
 
@@ -998,3 +1096,21 @@ error:
     if (operation)
         free(operation);
 }
+
+CallAudioMode cad_pulse_get_audio_mode(void)
+{
+    CadPulse *self = cad_pulse_get_default();
+    return self->audio_mode;
+}
+
+CallAudioSpeakerState cad_pulse_get_speaker_state(void)
+{
+    CadPulse *self = cad_pulse_get_default();
+    return self->speaker_state;
+}
+
+CallAudioMicState cad_pulse_get_mic_state(void)
+{
+    CadPulse *self = cad_pulse_get_default();
+    return self->mic_state;
+}
